using MetadataExtractor;
using MetadataExtractor.Formats.Exif;
using MetadataExtractor.Formats.QuickTime;
using Serilog;

namespace PhotoRenamer;

public static class MediaFileExtension
{
    public static DateTime GetDateTimeFromSourceFile(string file)
    {
        DateTime dateTime;
        try
        {
            var metadata = ImageMetadataReader.ReadMetadata(file);
            dateTime =
                GetDateTimeFromExif(metadata)
                ?? GetDateTimeFromMp4(metadata)
                ?? GetDateTimeFromLastWrite(file);
        }
        catch (ImageProcessingException e)
        {
            Log.Error(e, $"Error reading file information from {file}");
            dateTime = GetDateTimeFromLastWrite(file);
        }

        return dateTime;
    }

    private static DateTime GetDateTimeFromLastWrite(string file)
    {
        var creationTime = File.GetLastWriteTime(file);
        return creationTime;
    }

    private static DateTime? GetDateTimeFromExif(
        IEnumerable<MetadataExtractor.Directory> directories
    )
    {
        return
            directories
                .OfType<ExifIfd0Directory>()
                .FirstOrDefault()
                ?.TryGetDateTime(ExifDirectoryBase.TagDateTime, out var dateTime) == true
            ? dateTime
            : null;
    }

    private static DateTime? GetDateTimeFromMp4(
        IEnumerable<MetadataExtractor.Directory> directories
    )
    {
        return
            directories
                .OfType<QuickTimeMovieHeaderDirectory>()
                .FirstOrDefault()
                ?.TryGetDateTime(QuickTimeMovieHeaderDirectory.TagCreated, out var dateTime) == true
            ? dateTime
            : null;
    }
}
